<?php

namespace Drupal\entity_extender\Entity\Handler\Entity;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;

/**
 * The extended storage handler for entities.
 *
 * @see entity_extender_entity_type_alter()
 */
trait EntityStorageHandlerTrait {

  use EntityHandlerEntityExtenderManagerInjector;

  /**
   * {@inheritdoc}
   *
   * @throws \Exception
   */
  protected function doPreSave(EntityInterface $entity) {
    \assert($entity instanceof ContentEntityInterface);

    // Call parent now in order to populate the "$entity->original".
    $id = parent::doPreSave($entity);

    if ($extender = $this->getExtender($entity->getEntityTypeId(), $entity->bundle(), $entity)) {
      $extender->preSave();
    }

    return $id;
  }

  /**
   * {@inheritdoc}
   */
  protected function doPostSave(EntityInterface $entity, $update): void {
    \assert($entity instanceof ContentEntityInterface);

    parent::doPostSave($entity, $update);

    /* @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    if ($extender = $this->getExtender($entity->getEntityTypeId(), $entity->bundle(), $entity)) {
      $extender->postSave();
    }
  }

}
