<?php

namespace Drupal\entity_extender\Entity\Handler\Node;

use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\entity_extender\Entity\Handler\Entity\EntityHandlerEntityExtenderManagerInjector;
use Drupal\node\NodeAccessControlHandler;
use Drupal\node\NodeInterface;
use Drupal\entity_extender\Access\NodeAccessAllowedImmediately;
use Drupal\entity_extender\Plugin\EntityExtender\Entity\EntityAccessInterface;

/**
 * The extended access handler for nodes.
 *
 * @method EntityAccessInterface getExtender(string $entity_type, string $bundle, NodeInterface $entity)
 *
 * @see entity_extender_entity_type_alter()
 */
class NodeAccessHandler extends NodeAccessControlHandler {

  use EntityHandlerEntityExtenderManagerInjector;

  protected const SPECIFIC_INTERFACE = EntityAccessInterface::class;

  /**
   * {@inheritdoc}
   */
  public function access(EntityInterface $entity, $operation, AccountInterface $account = NULL, $return_as_object = FALSE) {
    \assert($entity instanceof NodeInterface);

    if ($extender = $this->getExtender($entity->bundle(), $entity)) {
      $account = $this->prepareUser($account);
      /* @var \Drupal\Core\Access\AccessResultInterface $access */
      $access = static::useMemo(function () use ($extender, $operation, $account): AccessResultInterface {
        return $extender->access($operation, $account);
      }, $extender, $operation, $account);

      // The "allowed" and "neutral" states let us proceed to core's access
      // checks. Only forbiddance terminates access checks immediately.
      if ($access->isForbidden()) {
        return $return_as_object ? $access : FALSE;
      }

      if ($access instanceof NodeAccessAllowedImmediately) {
        return $return_as_object ? $access : TRUE;
      }
    }

    return parent::access($entity, $operation, $account, $return_as_object);
  }

}
