<?php

namespace Drupal\entity_extender\Plugin\EntityExtender\Node;

use Drupal\Core\Entity\Display\EntityViewDisplayInterface;

/**
 * The interface to implement by a "node:CONTENT_TYPE" extender plugin.
 *
 * The purpose of this interface is to allow an extender providing
 * additional renderable data for the node.
 *
 * @see \Drupal\entity_extender\Entity\Handler\Node\NodeViewHandler::alterBuild()
 */
interface NodeViewInterface {

  /**
   * Alters the renderable data of a node.
   *
   * @param array $build
   *   The renderable data to modify.
   * @param \Drupal\Core\Entity\Display\EntityViewDisplayInterface $display
   *   The entity display.
   * @param string $view_mode
   *   The view mode an entity is to be rendered at.
   *
   * @see \Drupal\Core\Entity\EntityViewBuilder::alterBuild()
   */
  public function alterRenderableData(array &$build, EntityViewDisplayInterface $display, string $view_mode): void;

}
